from Tkinter import * #need Tkinter module for graphics
from math import * #need math module for floor, pi, cos, sin, etc.

#for canvas coordinate system
canvas_width=400
canvas_height=400

#for our x-y coordinate system
scale=pow(10,14) #in meters
xmin=-scale
xmax=scale
ymin=-scale
ymax=scale

#The canvas coordinate system is different from usual x-y coordinate system
#In the canvas system, the upper left corner is (0,0) and y points down.
#In our x-y system, y points up.
#In both systems, x points to the right.

#these two functions convert from our x-y system to the canvas system
def canvas_x(x, a=xmin, b=xmax, width=canvas_width):
    #floor rounds down to nearest whole number
    return (x - a) * width / (b - a)

def canvas_y(y, a=ymin, b=ymax, height=canvas_height):
    return (b - y) * height / (b - a)

degree=pi/180. #1 degree equals pi/180 radians

G=6.67*pow(10,-11) #universal graviational constant in m^3/(kg * s^2)
M=1.99*pow(10,30) #mass of sun in kg
GM=G*M #units m^3/s^2

class Simulation():
    """
    simulates sun's gravity acting on object
    rx0 is initial x position in meters
    ry0 is initial y position in meters
    vx0 is initial x velocity in m/s
    vy0 is initial y velocity in m/s
    dt is time increment size in seconds
    """
    def make_circle(self, cx, cy):
        rad=self.rad
        self.circle = canvas.create_oval(cx-rad,cy-rad,cx+rad,cy+rad,
                                         width=0,fill='red')
    def __init__(self, rx0, ry0, vx0, vy0, dt):
        self.vx = vx0
        self.vy = vy0
        self.rx = rx0
        self.ry = ry0
        self.dt = dt
        self.t = 0
        self.rad=10
        cx = canvas_x(rx0)
        cy = canvas_y(ry0)
        self.make_circle(cx, cy)
        #make sun
        canvas.create_oval(canvas_x(0)-self.rad,
                           canvas_y(0)-self.rad,
                           canvas_x(0)+self.rad,
                           canvas_y(0)+self.rad,
                           width=0,fill='yellow')
        self.advance()
    def advance(self):
        #print(self.rx,self.ry,self.vx,self.vy)
        #'c' for 'canvas'
        #'s' for 'start' of line segment
        cxs = canvas_x(self.rx)
        cys = canvas_y(self.ry)
        self.r=pow(self.rx*self.rx+self.ry*self.ry,1/2.)
        self.ax = -GM*self.rx/pow(self.r,3)
        self.ay = -GM*self.ry/pow(self.r,3)
        dvx = self.ax * self.dt
        dvy = self.ay * self.dt
        self.vx = self.vx + dvx
        self.vy = self.vy + dvy
        drx = self.vx * self.dt
        dry = self.vy * self.dt
        self.rx = self.rx + drx
        self.ry = self.ry + dry
        self.t = self.t + self.dt
        #'f' for 'finish' of line segment
        cxf = canvas_x(self.rx)
        cyf = canvas_y(self.ry)
        #make short line segment from old position (rx,ry) to
        #esimated new position (rx,ry) at time dt later.
        canvas.create_line(cxs, cys, cxf, cyf)
        canvas.delete(self.circle) #delete old circle
        self.make_circle(cxf, cyf) #replace with new circle
        #wait 1 milliseconds, then advance again
        canvas.after(int(1), self.advance)
        


#create window
master = Tk() 

#create 'canvas' that we draw lines on
canvas = Canvas(master, width=canvas_width, height=canvas_height, bg='white')

#actually displays canvas in window
canvas.pack()

Simulation(rx0 = 0,
           ry0 = -5*pow(10,12),
           vx0 = 7.1*pow(10,3),
           vy0 = 0,
           dt = pow(10,8))
    
    
    
#actually display window 
mainloop()
